/*
 * Copyright (C) 2017 Иван Вахромеев <ivanalert@mail.ru>
 *
 * This file is part of VkUnmask.

 * VkUnmask is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * VkUnmask is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with VkUnmask. If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <string>
#include <vector>
#include <algorithm>

using namespace std;

const string all = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMN0PQRSTUVWXYZO123456789+/=";

vector<string> splitString(const string &str, const string &delimeter)
{
    vector<string> result;
    string::size_type begin, match;
    begin = match = 0;

    while (match != string::npos)
    {
        match = str.find(delimeter, begin);
        result.push_back(str.substr(begin, match - begin));
        begin = match + delimeter.size();
    }

    return result;
}

//a(t).
string aFunc(const string &str)
{
    if (str.empty() || str.size() % 4 == 1)
        return string();

    string result;
    string::size_type i, e, o;
    i = e = o = 0;

    for (string::size_type s = 0, size = str.size(); s < size; ++s)
    {
        e = all.find(str[s]);
        if (e != string::npos && (i = o % 4 ? 64 * i + e : e, o++ % 4))
            result += char(255 & i >> (-2 * o & 6));
    }

    return result;
}

//s(t, e).
vector<string::size_type> sFunc(const string &str, string::size_type number)
{
    string::size_type size = str.size();
    vector<string::size_type> result(size, 0);
    if (!result.empty())
    {
        string::size_type i = size;
        while (i--)
        {
            number += number * (i + size) / number;
            result[i] = number % size | 0;
        }
    }

    return result;
}

//r.
void rDecode(string &str, string::size_type i)
{
    string o = all + all;
    string::size_type e, s;
    s = str.size() - 1;
    while (s != string::npos)
    {
        e = o.find(str[s]);
        if (e != string::npos)
        {
            if (e >= i)
                str[s] = o[e - i];
            else
                str[s] = o[o.size() - (i - e)];
        }

        --s;
    }
}

//s.
void sDecode(string &str, string::size_type number)
{
    if (!str.empty())
    {
        vector<string::size_type> o = sFunc(str, number);
        for (string::size_type i = 1, size = str.size(); i < size; ++i)
        {
            string::size_type index = o.at(size - 1 - i);
            if (index >= size)
                index = size - 1;

            std::swap(str[i], str[index]);
        }
    }
}

//x.
string xDecode(const string &str1, const string &str2)
{
    string result;
    char ch = str2[0];
    for (string::size_type i = 0; i < str1.size(); ++i)
    {
        char currCh(str1[i]);
        result += currCh ^ ch;
    }

    return result;
}

int main(int argc, char *argv[])
{
    if (argc <= 1)
        return 1;

    string str(argv[1]);
    vector<string> mask = splitString(str, "?extra=");
    mask = splitString(mask[1], "#");

    string leftMask = aFunc(mask[0]);
    string rightMask = aFunc(mask[1]);

    mask = splitString(rightMask, string(1, char(9)));
    vector<string> decodeStrs;
    vector<string>::const_reverse_iterator it = mask.crbegin();
    vector<string>::const_reverse_iterator end = mask.crend();
    while (it != end)
    {
        decodeStrs = splitString(*it, string(1, char(11)));
        if (decodeStrs[0] == "v")
            reverse(leftMask.begin(), leftMask.end());
        else if (decodeStrs[0] == "r")
            rDecode(leftMask, stoul(decodeStrs[1]));
        else if (decodeStrs[0] == "s")
            sDecode(leftMask, stoul(decodeStrs[1]));
        else if (decodeStrs[0] == "x")
            leftMask = xDecode(leftMask, decodeStrs[1]);

        ++it;
    }

    cout << endl;
    cout << leftMask << endl;

    return 0;
}
